;; -*- Scheme -*-

(include "eotya.scm")

;; interface

(define-predicate dictionary?)

;; get-item behaves just like 'assoc'.  it returns either a (key . value) pair,
;; or #f if the key is not present.
(define-operation (get-item self key))
(define-operation (set-item! self key value))
(define-operation (del-item! self key))
(define-operation (keys self))
(define-operation (values self))
(define-operation (items self))
(define-operation (size self))

;; implementation (using association lists, see SICP 3.3.3)

;; [how 'bout this: use an alist until we reach a certain size,
;;  then use an rbtree?]

(define (make-alist-dictionary)
  (let ((dict (list)))
    (object
     ((set-item! self key value)
      (let ((record (assq key dict)))
	(if record
	    (set-cdr! record value)
	    (set! dict (cons (cons key value) dict)))))
     ((del-item! self key)
      (set! dict 
	    (let loop ((l dict))
	      (cond ((null? l) l)
		    ((eq? (caar l) key)
		     (loop (cdr l)))
		    (else (cons (car l) (loop (cdr l))))))))
     ((get-item self key)
      (assq key dict))
     ((keys self)
      (map car dict))
     ((values self)
      (map cdr dict))
     ((items self)
      dict)
     ((size self)
      (length dict))
     )))

(define make-dictionary make-alist-dictionary)

