;; -*- Mode: Scheme -*-

(define (string-concat . strings)
  (letrec ((cat (lambda (strings)
		  (cond ((null? strings) "")
			((null? (cdr strings)) (car strings))
			(else (string-append
			       (car strings)
			       (cat (cdr strings))))))))
    (cat strings)))

;; make <n> connections to the local SMTP server,
;; say hello, goodbye, and then quit.

(define (smtp-test n)
  (let ((wheel (make-wheel 64 30.0)))
    (define (fnord)
      (let ((socket (make-chained-socket wheel AF_INET SOCK_STREAM 0)))
	(connect socket "127.0.0.1:25")
	(display (recv socket 1024))
	(send socket "HELO\r\n")
	(display (recv socket 1024))
	(send socket "QUIT\r\n")
	(display (recv socket 1024))
	(close socket)))
    ;; spawn connection[s] to the local SMTP server
    (let loop ((i n))
      (display "i ") (display i) (newline)
      (if (> i 0)
	  (begin 
	    (spawn wheel fnord)
	    (loop (- i 1)))))
    (spin wheel)
    ))

;; grab <url> from <addr> using <n> parallel connections.

(define (url-test addr url n)
  (let ((wheel (make-wheel 64 30.0)))
    (define (get-url addr url)
      (let ((socket (make-chained-socket wheel AF_INET SOCK_STREAM 0))
	    (request (string-concat "GET /" url " HTTP/1.0\r\n\r\n")))
	(connect socket addr)
	(send socket request)
	(let loop ((data (recv socket 1024)))
	  (if (string=? data "")
	      #f
	      (begin (display
		      (string-concat
		       (number->string (string-length data))
		       " bytes\n"))
		     (loop (recv socket 1024)))))
	(close socket)))
    (let loop ((i n))
      (if (> i 0)
	  (begin
	    (spawn wheel (lambda () (get-url addr url)))
	    (loop (- i 1)))))
    (spin wheel)
    ))

(define (echo-server addr)
  (let ((wheel (make-wheel 64 30.0)))
    ;; the server
    (define (server)
      (let ((socket (make-chained-socket wheel AF_INET SOCK_STREAM 0)))
	(bind socket addr)
	(listen socket 5)
	(let loop ((connection (accept socket))
		   (count 0))
	  (display (string-concat "client #" (number->string count) "\n"))
	  (spawn wheel (lambda () (client connection)))
	  (loop (accept socket) (+ count 1)))))

    ;; the client
    (define (client socket)
      (let loop ((data (recv socket 1024)))
	(cond ((string=? data "")
	       (close socket)
	       #f)
	      (else (send socket data)
		    (loop (recv socket 1024))))))

    ;; kick it off
    (spawn wheel server)
    (spin wheel)))

(define (http-test addr)
  (let ((wheel (make-wheel 64 30.0)))
    ;; the server
    (define (server)
      (let ((socket (make-chained-socket wheel AF_INET SOCK_STREAM 0)))
	(bind socket addr)
	(listen socket 5)
	(let loop ((connection (accept socket)))
	  (spawn wheel (lambda () (client connection)))
	  (loop (accept socket)))))

    (define canned-reply
      "Content-Type: text/html\r\n<html><h1>Hello, There!</h1></html>\r\n")

    ;; the client
    (define (client socket)
      ;; read the request
      (recv socket 1024)
      (send socket canned-reply)
      (close socket))

    ;; kick it off
    (spawn wheel server)
    (spin wheel)))

;(http-test (list-ref (argv) 1))
