;; -*- Scheme -*-
;;
;;	Author: Sam Rushing <rushing@nightmare.com>
;;	$Id: eotya.scm,v 1.2 1997/04/09 04:32:05 rushing Exp $
;;
;; a minimal YASOS for Gambit-C
;;
;; eotya (Enough Of The <Yet Another>'s)
;; yasos in terms of SLIB's 'object.scm' using gambit's 'define-macro'
;;
;; Usage: (include "eotya.scm")
;; 

(define-macro (define-predicate <name>)
  `(define ,<name> (make-generic-predicate)))

(define-macro (define-operation <arglist> . <body>)
  (let ((name (car <arglist>))
	(args (cdr <arglist>))
	(body
	 (if (null? <body>)
	     '(error "operation not handled")
	     <body>))
	)
    `(define ,name
       (make-generic-method
	(lambda ,args ,@body)))))

(define-macro (object . <methods-specs>)
  (let ((method-list
	 (map (lambda (x)
		(let ((method-args (car x))
		      (method-body (cdr x)))
		  `(make-method! self ,(car method-args)
				 (lambda ,(cdr method-args) ,@method-body))))
	  <methods-specs>)))
    `(let ((self (make-object)))
       ,@method-list
       self)))

(define-macro (object-with-ancestors <ancestor-specs> . <method-specs>)
  (let ((ancestor-names (map car <ancestor-specs>))
	(method-list
	 (map (lambda (x)
		(let ((method-args (car x))
		      (method-body (cdr x)))
		  `(make-method! self ,(car method-args)
				 (lambda ,(cdr method-args) ,@method-body))))
	      <method-specs>)))
    `(let* (,@<ancestor-specs>
	    (self (make-object ,@ancestor-names)))
       ,@method-list
       self)))

(define-macro (operate-as <component> <op> <composit> . <args>)
  `((get-method ,<component> ,<op>) ,<composit> ,@<args>))

;; debugging
;(define (macro-expand exp)
;  (let ((cte (##cte-top-cte ##interaction-cte)))
;    (if (pair? exp)
;	(let* ((first (car exp))
;	       (def (##macro-lookup cte first)))
;	  (if def
;	      (##macro-expand cte exp def)
;	      (error "no such macro")))
;	(error "say what?"))))
